/* This code implements the recording recipe for both MD-AMM and ME-AMM models
Reference Paper: Valle, M.E., A Class of Sparsely Connected Autoassociative Morphological Memories for Large Color Images,
IEEE Transactions on Neural Networks   */

#include "mex.h"
#include "stdlib.h"
#include "stdio.h"

int lexico(double u1, double u2, double u3, double v1, double v2, double v3) 
{
if (u1<v1)
	return -1;
else if (u1>v1)
	return +1;
else if (u2<v2)
	return -1;
else if (u2>v2)
	return +1;
else if (u3<v3)
	return -1;
else if (u3>v3)
	return +1;
else return 0;
}

/*----------------------------------------------------------------------*/
void mexFunction(int nlhs, mxArray *plhs[], 
    int nrhs, const mxArray *prhs[])

{
 if (nrhs != 2)
 	mexErrMsgTxt("The number of input arguments must be two.");

/* Get the name of the file in which the synaptic junctions will be stored. */
 char *filename;
 int buflen;
 buflen=mxGetNumberOfElements(prhs[0])+1;
 filename=(char *)mxCalloc(buflen,sizeof(char));
 mxGetString(prhs[0],filename,buflen);

/* Open the file where we will store the sparsely represented synaptic weight matrix J. */
 FILE *fp;
 if ((fp=fopen(filename,"wb+"))==NULL) 
	mexErrMsgTxt("The requested file could not be opened.");

 /* Find the dimension of the input matrix; */
 const *dimX=mxGetDimensions(prhs[1]); 
 
 if (dimX[1] != 3)
	mexErrMsgTxt("Error: The stored patterns must be color images (three columns).");

 unsigned long int N=dimX[0], K=dimX[2]; 
 
 double *X;
 X = mxGetPr(prhs[1]);

 /* Creating sparse matrices of indexes */
 unsigned long int nz=0, IndexI, IndexJ;
 unsigned long int i, j, xi, Nxi, Mxi;
 int L;

 /* The firt element of the sparsely represented synaptic weight matrix contains
    information about the size of stored patterns, i.e., J[1,1]=N. */ 
 fwrite(&N,sizeof(unsigned long int),1,fp);
 
 for (i=0; i<N; i++)
 	for (j=0; j<N; j++)
	{	
		L=-1;
		xi=0;
		while (xi<K && L<1)
		{
			Nxi=3*N*xi;
			L=lexico(X[j+Nxi],X[j+N+Nxi],X[j+2*N+Nxi],X[i+Nxi],X[i+N+Nxi],X[i+2*N+Nxi]);
			xi++;
		}
		if (L<1)
		{
			IndexI=i;
			IndexJ=j;
			fwrite(&IndexI,sizeof(unsigned long int),1,fp);
			fwrite(&IndexJ,sizeof(unsigned long int),1,fp);
			nz++;
		}
	}

 printf("Number of non-zeros elements: %lu \n",nz);
 fclose(fp);
}
