/* This code implements the MD-AMM model introduced at:
Valle, M.E., A Class of Sparsely Connected Autoassociative Morphological Memories for Large Color Images,
IEEE Transactions on Neural Networks   */

#include "mex.h"
#include "stdlib.h"
#include "stdio.h"

#define MINVALUE -1.0E20

int lexico(double u1, double u2, double u3, double v1, double v2, double v3) 
{
if (u1<v1)
	return -1;
else if (u1>v1)
	return +1;
else if (u2<v2)
	return -1;
else if (u2>v2)
	return +1;
else if (u3<v3)
	return -1;
else if (u3>v3)
	return +1;
else return 0;
}

/*----------------------------------------------------------------------*/
void mexFunction(int nlhs, mxArray *plhs[], 
    int nrhs, const mxArray *prhs[])

{
 if (nrhs != 2)
 	mexErrMsgTxt("The number of input arguments must be two.");

/* Get the name of the file in which the synaptic junctions was stored. */
 char *filename;
 int buflen;
 buflen=mxGetNumberOfElements(prhs[0])+1;
 filename=(char *)mxCalloc(buflen,sizeof(char));
 mxGetString(prhs[0],filename,buflen);

 /* Find the dimension of the input matrices; */
 const *dimx=mxGetDimensions(prhs[1]);
 
 if (dimx[1] != 3)
	mexErrMsgTxt("Error: The input pattern must be a color image (three columns).");
 
 double *x;
 x = mxGetPr(prhs[1]);

 /* Open the file where we will store the sparsely represented synaptic weight matrix J. */
 FILE *fp;
 if ((fp=fopen(filename,"rb"))==NULL) 
	mexErrMsgTxt("Failed to open the requested file.");

 // Read the number of lines of the output;
 unsigned long int N;
 fread(&N,sizeof(unsigned long int),1,fp);

 if (dimx[0] != N)
 	mexErrMsgTxt("Error: The input pattern does not agree with the dimensions specified on the file.");

 /* Create an mxArray for the output data */
 plhs[0] = mxCreateDoubleMatrix(N, 3, mxREAL);

 /* Create a pointer to the output data */
 double *y;
 y = mxGetPr(plhs[0]);
 
 // Setting y[i,j]=MINVALUE for all i and j;
 unsigned long k;
 for (k=0; k<N; k++)
 {
 	y[k]=MINVALUE;
	y[k+N]=MINVALUE;
	y[k+2*N]=MINVALUE;
 } 

 // Computing the output!!!
 unsigned long i, j;
 while (!feof(fp))
 {
	fread(&i,sizeof(unsigned long int),1,fp);
 	fread(&j,sizeof(unsigned long int),1,fp);
 	if (lexico(y[i],y[i+N],y[i+2*N],x[j],x[j+N],x[j+2*N])<0)
			{
				y[i]=x[j];
				y[i+N]=x[j+N];
				y[i+2*N]=x[j+2*N];
			}
 }

 fclose(fp);
}
